/*
  Magic Shadow Archiver for Linux - floppy.c

  This file is distributed under the GNU Public License, version 2 or at
  your option any later version. Read the file gpl.txt for details.

  Check for valid floppy disc geometry.
*/

#include <stdio.h>

#include "floppy.h"


/*-----------------------------------------------------------------------*/
/*
  Double-check information read from boot-sector as this is sometimes found to be incorrect
  The .ST image file should be divisible by the sector size & sectors per track
  NOTE - Pass information from boot-sector to this function (if we can't decide we leave it alone)
*/
void Floppy_DoubleCheckFormat(long DiscSize,unsigned short int *pnSides,unsigned short int *pnSectorsPerTrack)
{
  int nSectorsPerTrack;
  long TotalSectors;

  /* Now guess at number of sides */
  if (DiscSize<(500*1024))                      /* Is size is >500k assume 2 sides to disc! */
    *pnSides = 1;
  else
    *pnSides = 2;

  /* And Sectors Per Track (always 512 bytes per sector) */
  TotalSectors = DiscSize/512;                  /* # Sectors on disc image */
  /* Does this match up with what we've read from boot-sector? */
  nSectorsPerTrack = *pnSectorsPerTrack;
  if (nSectorsPerTrack==0)                      /* Check valid, default to 9 */
    nSectorsPerTrack = 9;
  if ((TotalSectors%nSectorsPerTrack)!=0) {
    /* No, we have an invalid boot-sector - re-calculate from disc size */
    if ((TotalSectors%9)==0)                    /* Work in this order.... */
      *pnSectorsPerTrack = 9;
    else if ((TotalSectors%10)==0)
      *pnSectorsPerTrack = 10;
    else if ((TotalSectors%11)==0)
      *pnSectorsPerTrack = 11;
    else if ((TotalSectors%12)==0)
      *pnSectorsPerTrack = 12;
  }
  /* else unknown, assume boot-sector is correct!!! */
}

/*-----------------------------------------------------------------------*/
/*
  Find details of disc image. We need to do this via a function as sometimes the boot-block
  is not actually correct with the image - some demos/game discs have incorrect bytes in the
  boot sector and this attempts to find the correct values.
*/
void Floppy_FindDiscDetails(unsigned char *pBuffer, int nImageBytes, unsigned short int *pnSectorsPerTrack,unsigned short int *pnSides)
{
  unsigned short int nSectorsPerTrack, nSides, nSectors;

  /* First do check to find number of sectors and bytes per sector */
  nSectorsPerTrack = (*(unsigned char *)(pBuffer+24)) | ((*(unsigned char *)(pBuffer+25))<<8);      /* SPT */
  nSides = (*(unsigned char *)(pBuffer+26)) | ((*(unsigned char *)(pBuffer+27))<<8);                 /* SIDE */
  nSectors = (*(unsigned char *)(pBuffer+19)) | ((*(unsigned char *)(pBuffer+20))<<8);               /* TOTAL SECTORS */

  /*fprintf(stderr, "1. nSectorsPerTrack=%i nSides=%i nSectors=%i\n", nSectorsPerTrack, nSides, nSectors);*/

  /* Now double-check info as boot-sector may contain incorrect information, eg 'Eat.st' demo, or single/double sides */
  if (nSectors != nImageBytes/NUMBYTESPERSECTOR)
    Floppy_DoubleCheckFormat(nImageBytes, &nSides, &nSectorsPerTrack);

  /*fprintf(stderr, "2. nSectorsPerTrack=%i nSides=%i\n", nSectorsPerTrack, nSides);*/

  /* And set values */
  if (pnSectorsPerTrack)
    *pnSectorsPerTrack = nSectorsPerTrack;
  if (pnSides)
    *pnSides = nSides;
}
