/*
  Magic Shadow Archiver for Linux - main.c

  This file is distributed under the GNU Public License, version 2 or at
  your option any later version. Read the file gpl.txt for details.
*/

#include <stdio.h>
#include <stdlib.h>
#include <endian.h>
#include <string.h>

#include "main.h"
#include "file.h"
#include "msa.h"



/*-----------------------------------------------------------------------*/
/*
  Swap 16-bit integer to/from 68000/PC format
*/
unsigned short int Swap68000Int(unsigned short int var)
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
 return((var<<8)|(var>>8));
#else
 return var;
#endif
}

/*-----------------------------------------------------------------------*/
/*
  Swap 32-bit integer to/from 68000/PC format
*/
unsigned long Swap68000Long(unsigned long var)
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
 return((var<<24)|((var<<8)&0x00FF0000)|((var>>8)&0x0000FF00)|(var>>24));
#else
 return var;
#endif
}



int main(int argc, char *argv[])
{
  long nImageBytes;
  int retval;
  unsigned char *discBuffer;
  char *sourceFileName;
  char *targetFileName;

  if(argc!=2 || argv[1][0]=='-')
  {
    printf("\rMagic Shadow (Un-)Archiver for Linux version 0.1.0.\n\n"
           "Usage:  %s FILENAME\n\n"
           "This program converts a MSA disc image into a ST disc image and vice versa.\n"
           "It is distributed under the GNU Public License, version 2 or at your\n"
           "option any later version. Please read the file gpl.txt for details.\n",
           argv[0]);
    return 0;
  }

  if(strlen(argv[1]) < 4)
  {
    fprintf(stderr,"Invalid file name %s\n", argv[1]);
    return -1;
  }

  sourceFileName = argv[1];
  targetFileName = malloc(strlen(sourceFileName) + 6);

  discBuffer = malloc(DRIVE_BUFFER_BYTES);

  if(discBuffer == NULL || targetFileName == NULL)
  {
    fprintf(stderr,"Not enough memory\n");
    return -1;
  }

  if(strcasecmp(&sourceFileName[strlen(sourceFileName)-3], "msa") == 0)
  {
    /* Convert MSA to ST disc image */

    /* Read the source disc image */
    nImageBytes = MSA_ReadDisc(sourceFileName, discBuffer);
    if(nImageBytes == 0 || nImageBytes > DRIVE_BUFFER_BYTES)
    {
      fprintf(stderr,"Could not read MSA disc!\n");
      return -1;
    }

    strcpy(targetFileName, sourceFileName);
    if(targetFileName[strlen(targetFileName)-4] == '.')
      targetFileName[strlen(targetFileName)-4] = 0;  /* Strip MSA extension */
    strcat(targetFileName, ".st");

    printf("Converting %s to %s (%li Bytes)\n", sourceFileName, targetFileName, nImageBytes);

    retval = File_Save(targetFileName, discBuffer, nImageBytes);
  }
  else
  {
    /* Convert ST to MSA disc image */
    nImageBytes = 0;

    /* Read the source disc image: Just load directly into buffer */
    File_Read(sourceFileName, discBuffer, &nImageBytes, NULL);
    if(nImageBytes == 0 || nImageBytes > DRIVE_BUFFER_BYTES)
    {
      fprintf(stderr,"Could not read ST disc!\n");
      return -1;
    }

    strcpy(targetFileName, sourceFileName);
    if(targetFileName[strlen(targetFileName)-3] == '.')
      targetFileName[strlen(targetFileName)-3] = 0;  /* Strip ST extension */
    strcat(targetFileName, ".msa");

    printf("Converting %s to %s (%li Bytes)\n", sourceFileName, targetFileName, nImageBytes);

    retval = MSA_WriteDisc(targetFileName, discBuffer, nImageBytes);
  }

  free(targetFileName);
  free(discBuffer);

  return retval;
}

